--************************************************************************************************
-- 8Kx16(16 KB) PM RAM for AVR Core(Xilinx)
-- Version 0.1
-- Designed by Ruslan Lepetenok 
-- Modified 29.10.2005
--************************************************************************************************

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

-- For Synplicity Synplify
--library virtexe;
--use	virtexe.components.all; 

-- Aldec
library	unisim;
use unisim.vcomponents.all;

entity XPM8Kx16 is port(
	                  cp2     : in  std_logic;
					  ce      : in  std_logic; 
	                  address : in  std_logic_vector(12 downto 0); 
					  din     : in  std_logic_vector(15 downto 0);		                
					  dout    : out std_logic_vector(15 downto 0);
					  weh     : in  std_logic;
					  wel     : in  std_logic
					  );
end XPM8Kx16;

architecture RTL of XPM8Kx16 is

type   RAMBlDOut_Type is array(2**(address'length-9)-1 downto 0) of  std_logic_vector(dout'range);
signal RAMBlDOut     : RAMBlDOut_Type;

signal WEBL     : std_logic_vector(2**(address'length-9)-1 downto 0);
signal WEBH     : std_logic_vector(2**(address'length-9)-1 downto 0);
signal gnd      : std_logic;
signal BRBusy   : std_logic_vector(2**(address'length-9)-1 downto 0);
signal BRBusy1  : std_logic_vector(2**(address'length-9)-1 downto 0);



COMPONENT ABRAMB4_S8
  PORT (
    clka : IN STD_LOGIC;
    rsta : IN STD_LOGIC;
    ena : IN STD_LOGIC;
    wea : IN STD_LOGIC_VECTOR(0 DOWNTO 0);
    addra : IN STD_LOGIC_VECTOR(8 DOWNTO 0);
    dina : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
    douta : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
    rsta_busy : OUT STD_LOGIC
  );
END COMPONENT;

begin

gnd <= '0';	

WEBH_Dcd:for i in WEBL'range generate 
 WEBL(i) <= '1' when (wel='1' and address(address'high downto 9)=i) else '0';
end generate ;

WEBL_Dcd:for i in WEBH'range generate 
 WEBH(i) <= '1' when (weh='1' and address(address'high downto 9)=i) else '0';
end generate ;

RAM_Inst:for i in 0 to 2**(address'length-9)-1 generate

RAM_ByteLow: component ABRAMB4_S8 port map(
                                      douta   => RAMBlDOut(i)(7 downto 0),
                                      addra => address(8 downto 0),
                                      dina   => din(7 downto 0),
                                      ena   => ce,
                                      clka  => cp2,
                                      wea   => WEBL(i downto i),
                                      rsta  => gnd,
                                      rsta_busy => BRBusy(i)
                                      );

RAM_ByteHigh: component ABRAMB4_S8 port map(
                                      douta   => RAMBlDOut(i)(15 downto 8),
                                      addra => address(8 downto 0),
                                      dina   => din(15 downto 8),
                                      ena   => ce,
                                      clka  => cp2,
                                      wea   => WEBH(i downto i),
                                      rsta  => gnd,
                                      rsta_busy => BRBusy1(i)                                      
                                      );
									  
end generate;

-- Output data mux
dout <= RAMBlDOut(CONV_INTEGER(address(address'high downto 9)));


end RTL;
